/*Copyright (C) 2020 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include "AbashinRuDbApi.h"


int get_pfs_day(char * filename, struct PFS ** pfs,
		time_t start_time, time_t stop_time) {
	int pfs_max = 0;
	how_much_pfs_record(filename, &pfs_max, start_time, stop_time);

	if (pfs_max < 2) return 1;

	if ((*pfs) == NULL) {
		(*pfs) = (struct PFS *) calloc(1, sizeof(struct PFS));
		(*pfs)->len = pfs_max;
		(*pfs)->pr = (struct PFS_RECORD *)
			calloc((*pfs)->len, sizeof(struct PFS_RECORD));
	}

	FILE * fl = fopen(filename, "r");
	if (fl == NULL) return 2;

	int pfs_count = 0;
	char buffer[100] = {0};
	while (!feof(fl)) {
		if (pfs_count >= pfs_max) break;
		if (fgets(buffer, 90, fl) == 0) continue;
		sscanf(buffer, "%ld %d %lf %d",
			&((*pfs)->pr[pfs_count].current_time),
			&((*pfs)->pr[pfs_count].intensity),
			&((*pfs)->pr[pfs_count].pfs),
			&((*pfs)->pr[pfs_count].device_id));
		if ((*pfs)->pr[pfs_count].current_time > stop_time)
			break;
		if ((*pfs)->pr[pfs_count].current_time > start_time)
			pfs_count++;
	}
	fclose(fl);
	(*pfs)->pr[pfs_count].current_time = 0;
	(*pfs)->pr[pfs_count].intensity = 0;
	(*pfs)->pr[pfs_count].pfs = 0.0;
	(*pfs)->pr[pfs_count].device_id = 0;
	return 0;
}

int get_etalon(char * etalon_filename, struct MOUSE_ETALON ** me,
					struct KEYBOARD_ETALON ** ke) {
	int keyboard_press_max = 0;
	int mouse_press_max = 0;
	how_much_etalon_record(etalon_filename , &mouse_press_max, &keyboard_press_max);
	if ((!keyboard_press_max) && (!mouse_press_max)) return 2;

	*me = (struct MOUSE_ETALON *) calloc(1, sizeof(struct MOUSE_ETALON));
	(*me)->len = mouse_press_max;
	(*me)->mer = (struct MOUSE_ETALON_RECORD *)
		calloc((*me)->len, sizeof(struct MOUSE_ETALON_RECORD));
	*ke = (struct KEYBOARD_ETALON *) calloc(1, sizeof(struct KEYBOARD_ETALON));
	(*ke)->len = keyboard_press_max;
	(*ke)->ker = (struct KEYBOARD_ETALON_RECORD *)
		calloc((*ke)->len, sizeof(struct KEYBOARD_ETALON_RECORD));

	FILE * fl = fopen(etalon_filename, "r");
	if (!fl) return 3;
	time_t current_time = 0;
	if (fscanf(fl, "%ld", &current_time) == 0) {
		fclose(fl);
		return 4;
	}
	int k_press_count = 0;
	int m_press_count = 0;

	while (!feof(fl)) {
		int k_or_m = 0;
		if (fscanf(fl, "%d", &k_or_m) == 0) {
			fclose(fl);
			return 0;
		}
		if (k_or_m == 1) {
			if (fscanf(fl, "%d %d %d %d %d %d %d %d",
				&(*ke)->ker[k_press_count].key1,
				&(*ke)->ker[k_press_count].retaining1_min,
				&(*ke)->ker[k_press_count].retaining1_max,
				&(*ke)->ker[k_press_count].space1_min,
				&(*ke)->ker[k_press_count].space1_max,
				&(*ke)->ker[k_press_count].key2,
				&(*ke)->ker[k_press_count].retaining2_min,
				&(*ke)->ker[k_press_count].retaining2_max)
				== 0) { fclose(fl); return 0; }
			k_press_count++;
		}
		else {
			if (fscanf(fl, "%d %d %d",
				&(*me)->mer[m_press_count].key,
				&(*me)->mer[m_press_count].retaining1_min,
				&(*me)->mer[m_press_count].retaining1_max)
				== 0) { fclose(fl); return 0; }
			m_press_count++;
		}
	}
	fclose(fl);
	return 0;
}

void how_much_pfs_record(char * filename, int * pfs_max,
				time_t start_time, time_t stop_time) {
	*pfs_max = 1;
	struct PFS_RECORD pfsr = {0};
	FILE * fl = fopen(filename, "r");
	if (!fl) return;
	char buffer[100] = {0};
	while (!feof(fl)) {
		if (fgets(buffer, 90, fl) == 0) continue;
		sscanf(buffer, "%ld %d %lf %d", &pfsr.current_time,
			&pfsr.intensity, &pfsr.pfs, &pfsr.device_id);
		if (pfsr.current_time > stop_time) break;
		if (pfsr.current_time > start_time) {
#pragma GCC diagnostic ignored "-Wunused-value"
			*pfs_max += 1;
#pragma GCC diagnostic pop
		}
	}
	fclose(fl);
}

void how_much_etalon_record(char * etalon_filename, int * mouse_press_max,
					int * keyboard_press_max) {
	*mouse_press_max = *keyboard_press_max = 0;
	FILE * fl = fopen(etalon_filename, "r");
	char * buffer = calloc(200, sizeof(char));
	if (fl) {
		memset(buffer, 0, sizeof(char) * 200);
		if (fgets(buffer, 200, fl) == NULL) {
			free(buffer);
			fclose(fl);
			return;
		}
		int k_or_m = 0;
		while (fgets(buffer, 200, fl) != NULL) {
			sscanf(buffer, "%d", &k_or_m);
#pragma GCC diagnostic ignored "-Wunused-value"
			if (k_or_m == 1) (*keyboard_press_max)++;
			else (*mouse_press_max)++;
#pragma GCC diagnostic pop
		}
		fclose(fl);
	}
	free(buffer);
}

int add_log_pfs(struct PFS_RECORD * pfsr, char * filename) {
	FILE * fl = fopen(filename, "a");
	if (fl) {
		fprintf(fl, "%ld %d %lf %d\n", pfsr->current_time,
			pfsr->intensity, pfsr->pfs,
			pfsr->device_id);
		fclose(fl);
	}
	else return 1;
	return 0;
}

int rewrite_current_pfs(struct PFS_RECORD * pfsr, char * filename) {
	FILE * fl = fopen(filename, "w");
	if (fl) {
		fprintf(fl, "%ld %d %lf %d", pfsr->current_time,
			pfsr->intensity, pfsr->pfs, pfsr->device_id);
		fclose(fl);
	}
	else return 1;
	return 0;
}

int get_last_pfs(struct PFS_RECORD * pfsr, char * filename) {
	FILE * fl = fopen(filename, "r");
	if (fl) {
		char * buffer = calloc(200, sizeof(char));
		if (fgets(buffer, 90, fl) == 0) {
			free(buffer);
			fclose(fl);
			return 1;
		}
		sscanf(buffer, "%ld %d %lf %d", &pfsr->current_time,
			&pfsr->intensity, &pfsr->pfs, &pfsr->device_id);
		free(buffer);
		fclose(fl);
	}
	else { fclose(fl); return 1; };
	return 0;
}

int save_etalon (char * filename_temp, char * filename,
		struct MOUSE_ETALON * me, struct KEYBOARD_ETALON * ke) {
	int how_many_m = 0;
	int how_many_k = 0;

	FILE * efile = fopen(filename, "w");
	if (efile == NULL) return 1;
	fprintf(efile, "%ld\n", time(NULL));

	int count = 0;
	for (count = 0; count < me->len; count++) {
		if (me->mer[count].key != 0 &&
			me->mer[count].retaining1_min < me->mer[count].retaining1_max) {
			fprintf(efile, "2 %d %d %d\n", me->mer[count].key,
				me->mer[count].retaining1_min,
				me->mer[count].retaining1_max);
			how_many_m++;
		}
		else break;
	}

	for (count = 0; count < ke->len; count++) {
		if (ke->ker[count].key1 != 0 && ke->ker[count].key2 != 0 &&
			ke->ker[count].retaining1_min < ke->ker[count].retaining1_max &&
			ke->ker[count].space1_min < ke->ker[count].space1_max &&
			ke->ker[count].retaining2_min < ke->ker[count].retaining2_max) {
			fprintf(efile, "1 %d %d %d %d %d %d %d %d\n",
				ke->ker[count].key1, ke->ker[count].retaining1_min, ke->ker[count].retaining1_max, ke->ker[count].space1_min, ke->ker[count].space1_max, ke->ker[count].key2, ke->ker[count].retaining2_min, ke->ker[count].retaining2_max);
			how_many_k++;
		}
	}
	fclose(efile);

	if (how_many_k > 10) {
		char * buff = calloc(1, sizeof(char) * 1000);
		sprintf(buff, "mv -f %s %s", filename_temp, filename);
		int result = system(buff);
		if (result) fprintf(stderr, "DEBUG: Can`t move %s", filename_temp);
		free(buff);
	}

	char * buff = calloc(1, sizeof(char) * 1000);
	sprintf(buff, "rm -f %s", filename_temp);
	int result = system(buff);
	if (result)
		fprintf(stderr, "DEBUG: Can`t remove %s", filename_temp);
	free(buff);

	return 0;
}

int get_input_device_data(char * savedata, struct MOUSE_PRESS * mp,
		struct KEYBOARD_PRESS * kp, time_t t_start, time_t t_stop) {
	if (t_start != 0 && t_start >= t_stop) return 1;
	int code = 0;
	int previous_code = 0;
	int intensity = 0;
	time_t current_time = 0;
	time_t start_time = 0;

	int count_ke = 0;
	int count_me = 0;

	FILE * dfile = fopen(savedata, "r");
	if (fscanf(dfile, "%d %ld", &code, &start_time) == 0)
		{ fclose(dfile); return 2; }
	current_time = start_time;
	previous_code = code;
	while (!feof(dfile)) {
		if (fscanf(dfile, "%d", &code) == 0)
			{ fclose(dfile); return 0; }
		if (code == 0) {
			if (fscanf(dfile, "%ld", &current_time) == 0)
				{ fclose(dfile); return 0; }
			char endline = 0;
			if (fscanf(dfile, "%c", &endline) == 0)
				{ fclose(dfile); return 0; }
		}
		if (code == 1)
			if (code != previous_code) {
				if (fscanf(dfile, "%d", &intensity) == 0)
					{ fclose(dfile); return 0; }
			}
			else {
				if (fscanf(dfile, "%d", &kp->kpr[count_ke].intensity) == 0)
					{ fclose(dfile); return 0; }
				if (!dfile) {
					previous_code = code;
					continue;
				}
				if (fscanf(dfile, "%d", &kp->kpr[count_ke].key1) == 0)
					{ fclose(dfile); return 0; }
				if (!dfile) {
					previous_code = code;
					continue;
				}
				if (fscanf(dfile, "%d", &kp->kpr[count_ke].retaining1) == 0)
					{ fclose(dfile); return 0; }
				if (!dfile) {
					previous_code = code;
					continue;
				}
				if (fscanf(dfile, "%d", &kp->kpr[count_ke].space1) == 0)
					{ fclose(dfile); return 0; }
				if (!dfile) {
					previous_code = code;
					continue;
				}
				if (fscanf(dfile, "%d", &kp->kpr[count_ke].key2) == 0)
					{ fclose(dfile); return 0; }
				if (!dfile) {
					previous_code = code;
					continue;
				}
				if (fscanf(dfile, "%d", &kp->kpr[count_ke].retaining2) == 0)
					{ fclose(dfile); return 0; }
				if (!dfile) {
					previous_code = code;
					continue;
				}
				kp->kpr[count_ke].current_time = current_time;
				if (!dfile) {
					previous_code = code;
 					continue;
				}
				if (t_start != 0 &&
				kp->kpr[count_ke].current_time < t_start &&
				kp->kpr[count_ke].current_time > t_stop) {
					kp->kpr[count_ke].current_time = 0;
					kp->kpr[count_ke].intensity = 0;
					kp->kpr[count_ke].key1 = 0;
					kp->kpr[count_ke].retaining1 = 0;
					kp->kpr[count_ke].space1 = 0;
					kp->kpr[count_ke].key2 = 0;
					kp->kpr[count_ke].retaining2 = 0;
				}
				else {
					if(count_ke++ > kp->len)
				 		{ fclose(dfile); return 0; }
				 }
			}
			else;
		if (code == 2)
			if (code != previous_code) {
				if (fscanf(dfile, "%d", &intensity) == 0) { fclose(dfile); return 0; }
			}
			else {
				if (fscanf(dfile, "%d", &mp->mpr[count_me].intensity) == 0)
					{ fclose(dfile); return 0; }
				if (!dfile) {
					previous_code = code;
					continue;
				}
				if (fscanf(dfile, "%d", &mp->mpr[count_me].key) == 0)
					{ fclose(dfile); return 0; }
				if (!dfile) {
					previous_code = code;
					continue;
				}
				if (fscanf(dfile, "%d", &mp->mpr[count_me].retaining1) == 0)
					{ fclose(dfile); return 0; }
				if (!dfile) {
					previous_code = code;
					continue;
				}
				mp->mpr[count_me].current_time = current_time;
				if (!dfile) {
					previous_code = code;
					continue;
				}
				if (t_start != 0 &&
				mp->mpr[count_me].current_time < t_start &&
				mp->mpr[count_me].current_time > t_stop) {
					mp->mpr[count_me].current_time = 0;
					mp->mpr[count_me].intensity = 0;
					mp->mpr[count_me].key = 0;
					mp->mpr[count_me].retaining1 = 0;
				}
				else {
					if(count_me++ > mp->len)
				 		return 0;
				 }
			}
		else;
		previous_code = code;
	}
	fclose(dfile);
	return 0;
}
