/*Copyright (C) 2020 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include "AbashinRuEtalon.h"

void create_etalon(int massive_size, char * etalondata,
					char * etalon_filename) {
	struct MOUSE_PRESS * mp = calloc(1, sizeof(struct MOUSE_PRESS));
	mp->len = massive_size;
	mp->mpr = calloc(mp->len, sizeof(struct MOUSE_PRESS_RECORD));

	struct KEYBOARD_PRESS * kp = calloc(1, sizeof(struct KEYBOARD_PRESS));
	kp->len = massive_size;
	kp->kpr = calloc(kp->len, sizeof(struct KEYBOARD_PRESS_RECORD));

	get_input_device_data(etalondata, mp, kp, 0, 0);

	if(calculate_and_save_etalon(mp,
		MIN_INTENSITY_MOUSE, kp, MIN_INTENSITY_KEYBOARD,
		massive_size, etalon_filename))
		puts ("Message: Etalon don`t save.");

	free(mp->mpr);
	free(mp);
	free(kp->kpr);
	free(kp);
}

void calc_mouse_etalon(struct MOUSE_PRESS * mp, 
	struct MOUSE_ETALON * me, int min_intensity) {
	int m_device_count = 0;
	int glob_count = 0;
	for (glob_count = 0; glob_count < mp->len; glob_count++) {
		if (mp->mpr[glob_count].current_time != 0) {
			int intensity = 0;
			int value = 0;
			me->mer[m_device_count].key = mp->mpr[glob_count].key;
			me->mer[m_device_count].retaining1_max =
				me->mer[m_device_count].retaining1_min =
					mp->mpr[glob_count].retaining1;
			int second_count = glob_count + 1;
			for ( ; second_count < me->len; second_count++) {
				if (me->mer[m_device_count].key == mp->mpr[second_count].key) {
					if (mp->mpr[second_count].intensity != 0 &&
					mp->mpr[second_count].retaining1 < 1000) {
						value += mp->mpr[second_count].retaining1 *
						mp->mpr[second_count].intensity;
						intensity += mp->mpr[second_count].intensity;
						if (me->mer[m_device_count].retaining1_min > mp->mpr[second_count].retaining1)
							me->mer[m_device_count].retaining1_min = mp->mpr[second_count].retaining1;
					}
					mp->mpr[second_count].current_time = 0;
					mp->mpr[second_count].intensity = 0;
					mp->mpr[second_count].key = 0;
					mp->mpr[second_count].retaining1 = 0;
				}
			}
			if (intensity > min_intensity) {
				int average = value / intensity;
				me->mer[m_device_count].retaining1_min = me->mer[m_device_count].retaining1_min + (average - me->mer[m_device_count].retaining1_min)/2;
				me->mer[m_device_count].retaining1_max = average + (average - me->mer[m_device_count].retaining1_min);
				m_device_count++;
			}
		}
	}
}

void calc_keyboard_etalon(struct KEYBOARD_PRESS * kp,
	struct KEYBOARD_ETALON * ke, int min_intensity) {

	int k_device_count = 0;
	int glob_count = 0;

	for (glob_count = 0; glob_count < kp->len; glob_count++) {
		if (kp->kpr[glob_count].current_time != 0) {
			int intensity = 0;
			int value_retaining1 = 0;
			int value_space1 = 0;
			int value_retaining2 = 0;
			ke->ker[k_device_count].key1 =
				kp->kpr[glob_count].key1;
			ke->ker[k_device_count].key2 =
				kp->kpr[glob_count].key2;
			ke->ker[k_device_count].retaining1_max =
				ke->ker[k_device_count].retaining1_min =
					kp->kpr[glob_count].retaining1;
			ke->ker[k_device_count].space1_max =
				ke->ker[k_device_count].space1_min =
					kp->kpr[glob_count].space1;
			ke->ker[k_device_count].retaining2_max =
				ke->ker[k_device_count].retaining2_min =
					kp->kpr[glob_count].retaining2;
			int second_count = glob_count + 1;

			for ( ; second_count < ke->len; second_count++) {
				if (ke->ker[k_device_count].key1 ==
				kp->kpr[second_count].key1 &&
				ke->ker[k_device_count].key2 ==
				kp->kpr[second_count].key2) {
					if (kp->kpr[second_count].intensity != 0 &&
					kp->kpr[second_count].retaining1 < 1000 &&
					kp->kpr[second_count].space1 < 1000 &&
					kp->kpr[second_count].retaining2 < 1000) {
						value_retaining1 +=
						kp->kpr[second_count].retaining1 * 
						kp->kpr[second_count].intensity;
						value_space1 +=
						kp->kpr[second_count].space1 *
						kp->kpr[second_count].intensity;
						value_retaining2 +=
						kp->kpr[second_count].retaining2 *
						kp->kpr[second_count].intensity;
						intensity += 
						kp->kpr[second_count].intensity;
						if (ke->ker[k_device_count].retaining1_min >
						kp->kpr[second_count].retaining1 ||
						ke->ker[k_device_count].retaining1_min == 0)
							ke->ker[k_device_count].retaining1_min = 
							kp->kpr[second_count].retaining1;
						if (ke->ker[k_device_count].space1_min >
						kp->kpr[second_count].space1 ||
						ke->ker[k_device_count].space1_min == 0)
							ke->ker[k_device_count].space1_min = 
							kp->kpr[second_count].space1;
						if (ke->ker[k_device_count].retaining2_min >
						kp->kpr[second_count].retaining2 ||
						ke->ker[k_device_count].retaining2_min == 0)
							ke->ker[k_device_count].retaining2_min = 
							kp->kpr[second_count].retaining2;
					}
					kp->kpr[second_count].current_time = 0;
					kp->kpr[second_count].intensity = 0;
					kp->kpr[second_count].key1 = 0;
					kp->kpr[second_count].retaining1 = 0;
					kp->kpr[second_count].space1 = 0;
					kp->kpr[second_count].key2 = 0;
					kp->kpr[second_count].retaining2 = 0;
				}
			}

			if (intensity > min_intensity) {
				int average_retaining1 = 0;
				int average_space1 = 0;
				int average_retaining2 = 0;

				average_retaining1 = value_retaining1 / intensity;
				average_space1 = value_space1 / intensity;
				average_retaining2 = value_retaining2 / intensity;

				ke->ker[k_device_count].retaining1_min =
					ke->ker[k_device_count].retaining1_min +
					(average_retaining1 - ke->ker[k_device_count].retaining1_min)/2;
				ke->ker[k_device_count].space1_min =
					ke->ker[k_device_count].space1_min +
					(average_space1 - ke->ker[k_device_count].space1_min)/2;
				ke->ker[k_device_count].retaining2_min =
					ke->ker[k_device_count].retaining2_min +
					(average_retaining2 - ke->ker[k_device_count].retaining2_min)/2;
				ke->ker[k_device_count].retaining1_max =
					average_retaining1 +
					(average_retaining1 - ke->ker[k_device_count].retaining1_min);
				ke->ker[k_device_count].space1_max =
					average_space1 +
					(average_space1 - ke->ker[k_device_count].space1_max);
				ke->ker[k_device_count].retaining2_max =
					average_retaining2 +
					(average_retaining2 - ke->ker[k_device_count].retaining2_max);

				k_device_count++;
			}
		}
	}
}

int calculate_and_save_etalon (struct MOUSE_PRESS * mp, int min_intensity_mouse,
		struct KEYBOARD_PRESS * kp, int min_intensity_keyboard,
		int massive_size, char * etalon_filename){

	struct MOUSE_ETALON * me = 
		calloc(1, sizeof(struct MOUSE_ETALON));
	me->len = massive_size;
	me->mer = calloc(me->len, sizeof(struct MOUSE_ETALON_RECORD));
	struct KEYBOARD_ETALON * ke = 
		calloc(1, sizeof(struct KEYBOARD_ETALON));
	ke->len = massive_size;
	ke->ker = calloc(ke->len, sizeof(struct KEYBOARD_ETALON_RECORD));

	calc_mouse_etalon(mp, me, min_intensity_mouse);
	calc_keyboard_etalon(kp, ke, min_intensity_keyboard);

	if (save_etalon (ETALONTEMP, etalon_filename, me, ke))
		puts("Warning: can`t save etalons.");

	free(me->mer);
	free(me);
	free(ke->ker);
	free(ke);

	return 0;  
}
