/*Copyright (C) 2020 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include "AbashinRuReport.h"

char * switch_color(char * buffer, time_t t_delta, double pfs) {
	int t_reiting = 0;
	if (t_delta / 60 < 100) t_reiting = 3;
	else if (t_delta / 60 < 120) t_reiting = 2;
	else t_reiting = 1;

	int pfs_reiting = 0;
	if (pfs > 0.5) pfs_reiting = 3;
	else if (pfs > 0.4) pfs_reiting = 2;
	else  pfs_reiting = 1;

	if (t_reiting < pfs_reiting) pfs_reiting = t_reiting;

	switch(pfs_reiting) {
		case 3:
			strcpy(buffer, "green");
			break;
		case 2:
			strcpy(buffer, "yellow");
			break;
		default:
			strcpy(buffer, "red");
	}

	return buffer;
}

char * create_report_string(struct REPORT_RECORD * rep_rec, char * report_string) {
	report_string[0] = 0;
	char text_color[7] = {0};
	char convert_buff[100] = {0};
	strcat(report_string, "<div class=\"row\">");

	strcat(report_string, "<span class=\"cell\" style=\"text-align:center;color:");
	if (rep_rec->rest_or_work == 0)
		strcpy(text_color, "black");
	else
		strcpy(text_color, switch_color(convert_buff, rep_rec->stop_time -
			rep_rec->stop_time, rep_rec->pfs));
	strcat(report_string, text_color);
	strcat(report_string, ";\">");
	struct tm * timeinfo;
	timeinfo = localtime(&rep_rec->start_time);
	sprintf(convert_buff, "%d.%d.%d %d:%d.%d", timeinfo->tm_mday,
			timeinfo->tm_mon+1, timeinfo->tm_year + 1900,
			timeinfo->tm_hour, timeinfo->tm_min,
			timeinfo->tm_sec);
	strcat(report_string, convert_buff);
	strcat(report_string, "</span>");

	strcat(report_string, "<span class=\"cell\" style=\"text-align:center;color:");
	strcat(report_string, text_color);
	strcat(report_string, ";\">");
	timeinfo = localtime(&rep_rec->stop_time);
	sprintf(convert_buff, "%d.%d.%d %d:%d.%d", timeinfo->tm_mday,
			timeinfo->tm_mon + 1, timeinfo->tm_year + 1900,
			timeinfo->tm_hour, timeinfo->tm_min,
			timeinfo->tm_sec);
	strcat(report_string, convert_buff);
	strcat(report_string, "</span>");

	strcat(report_string, "<span class=\"cell\" style=\"text-align:center;color:black;");
	if (strcmp(text_color, "black")) {
		strcat(report_string, "background-color:");
		strcat(report_string, text_color);
		strcat(report_string, ";");
	}
	strcat(report_string, "\">");
	if (rep_rec->pfs_count > 0)
		sprintf(convert_buff, "%d%%", (int)(rep_rec->pfs * 100.0));
	else
		strcpy(convert_buff, "undefined");
	if (rep_rec->rest_or_work == 1)
		strcat(report_string, convert_buff);
	else
		strcat(report_string, "-");
	strcat(report_string, "</span>");

	strcat(report_string, "<span class=\"cell\" style=\"text-align:center;color:");
	strcat(report_string, text_color);
	strcat(report_string, ";\">");
	if (rep_rec->intensity) {
		sprintf(convert_buff, "%d", rep_rec->intensity);
		strcat(report_string, convert_buff);
	}
	else
		strcat(report_string, "-");
	strcat(report_string, "</span>");

	strcat(report_string, "<span class=\"cell\" style=\"text-align:center;color:");
	strcat(report_string, text_color);
	strcat(report_string, ";\">");
	if (rep_rec->pfs_count) {
		sprintf(convert_buff, "%d", rep_rec->pfs_count);
		strcat(report_string, convert_buff);
	}
	else
		strcat(report_string, "-");
	strcat(report_string, "</span>");

	strcat(report_string, "<span class=\"cell\" style=\"text-align:center;color:");
	strcat(report_string, text_color);
	strcat(report_string, ";\">");
	if (rep_rec->stop_time - rep_rec->start_time > 0) {
		strcat(report_string, min_to_hm(rep_rec->stop_time -
			rep_rec->start_time, convert_buff));
	}
	else
		strcat(report_string, "-");
	strcat(report_string, "</span>");

	strcat(report_string, "<span class=\"cell\" style=\"text-align:center;color:");
	strcat(report_string, text_color);
	strcat(report_string, ";\">");
	if (rep_rec->rest_or_work == 1) {
		if (rep_rec->intensity > 10)
			strcat(report_string, MESSAGETYPEOUTPUT);
		else strcat(report_string, MESSAGETYPEINPUT);
	}
	else strcat(report_string, MESSAGETYPEREST);
	strcat(report_string, "</span>");

	strcat(report_string, "</div>");

	return report_string;
}

void print_report (struct PFS * pfs, char * filename,
					time_t current_time) {
	FILE * f_rep = fopen(filename, "w");
	fputs("<!DOCTYPE html>", f_rep);
	fputs("<html lang=\"ru\">", f_rep);
	fputs("<head>", f_rep);
	fputs("<meta charset=\"utf-8\">", f_rep);
	fputs("<title>Abashin.ru</title>", f_rep);
	fputs("<style type=\"text/css\">", f_rep);
	fputs("h1 { color: #aa0000 }", f_rep);
	fputs("#table {", f_rep);
	fputs("display: table;", f_rep);
	fputs("width:100%;", f_rep);
	fputs("border:1px solid white;", f_rep);
	fputs("border-collapse:collapse;", f_rep);
	fputs("}", f_rep);
	fputs(".row {", f_rep);
	fputs("display: table-row;", f_rep);
	fputs("border:1px solid white;", f_rep);
	fputs("}", f_rep);
	fputs(".cell {", f_rep);
	fputs("display: table-cell;", f_rep);
	fputs("border:1px solid white;", f_rep);
	fputs("}", f_rep);
	fputs("</style>", f_rep);
	fputs("</head>", f_rep);
	fputs("<body>", f_rep);
	fputs("<script> function fresh() { location.reload(); } setInterval(\"fresh()\",60000); </script>", f_rep);
	fputs("<h1 align = center>Current status</h1>", f_rep);

	if (pfs == NULL || pfs->len == 0) {
		fputs("</body>", f_rep);
		fputs("</html>", f_rep);
		return;
	}

	fputs("<p style=\"font-size: 200%; text-align: center;\">Poductivity: <font style=\"color:", f_rep);

	double pfs_local = 0.0;
	if (pfs->len > 2) pfs_local = pfs->pr[pfs->len-2].pfs;
	int time_break = when_a_break (pfs, current_time);
	if (time_break > 88) pfs_local = 0.0;
	if (pfs_local > 0.50) fprintf(f_rep, "green");
	else if (pfs_local < 0.35) fprintf(f_rep, "red");
	else fprintf(f_rep, "yellow");
	fputs(";\">", f_rep);
	if (pfs_local > 0.0)
		fprintf(f_rep, "%d%%", (int)(pfs_local*100.0));
	else
		fprintf(f_rep, "undefined");
	fputs(".</font><br>Before rest: <font style=\"color:", f_rep);
	if (time_break >= 5) fprintf(f_rep, "green");
	else if (time_break < -5) fprintf(f_rep, "red");
	else fprintf(f_rep, "yellow");
	if (time_break < 0) time_break = 0;
	fprintf(f_rep, ";\">%d min.</font><br>Worked out for the day: <font style=\"color:", time_break);
	int time_work = 0;
	int time_rest = 0;
	int count = 0;
	for (count = 0; count < pfs->len; count++) {
		if (pfs->pr[count].intensity > 0) {
			if ((time_rest < 9) && (time_rest > 0)) {
				time_work += time_rest + 1;
				time_rest = 0;
			}
			else time_work++;
		}
		else {
			time_rest++;
		}
	}
	if (time_rest < 10) time_work += time_rest;

	if (time_work <= 450) fprintf(f_rep, "green");
	else if (time_work >= 480) fprintf(f_rep, "red");
	else fprintf(f_rep, "yellow");
	fputs(";\">", f_rep);
	fprintf(f_rep, "%d h. %d min.</font></p>\n", time_work/60, time_work%60);
	fputs("<p>&nbsp;</p>", f_rep);
	fputs("<h1 align = center>Daily report</h1>", f_rep);
	fputs("<div id=\"table\">", f_rep);
	fputs("<div class=\"row\">", f_rep);
	fputs("<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Start</span>", f_rep);
	fputs("<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Stop</span>", f_rep);
	fputs("<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Productivity</span>", f_rep);
	fputs("<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Intensity</span>", f_rep);
	fputs("<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Active min.</span>", f_rep);
	fputs("<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Time interval</span>", f_rep);
	fputs("<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Type of work</span>", f_rep);
	fputs("</div>", f_rep);

	struct REPORT * rep = NULL;
	pfs_to_report_record (pfs, &rep);

	char * report_string = calloc(5000, sizeof(char));
	for (count = rep->len-1; count >= 0; count--) {
		if (rep->rr[count].start_time == 0) continue;
		fputs(create_report_string(&(rep->rr[count]),
			report_string), f_rep);
	}
	free(report_string);

	fputs("</div>", f_rep);
	fputs("</body>", f_rep);
	fputs("</html>", f_rep);
	fclose(f_rep);
}

void pfs_to_report_record (struct PFS * pfs,struct REPORT **rep) {
	*rep = calloc(1, sizeof(struct REPORT));
	(*rep)->len = 1440;
	(*rep)->rr = calloc((*rep)->len, sizeof(struct REPORT_RECORD));

	int count = 0;
	int count_rr = 0;

	for (count = 0; count < pfs->len; count++) {
		if (!pfs->pr[count].current_time) continue;
		if (count_rr == 0) {
			(*rep)->rr[count_rr].start_time =
				pfs->pr[count].current_time - 60;
			if (pfs->pr[count].current_time)
				(*rep)->rr[count_rr].stop_time =
				pfs->pr[count].current_time;
			(*rep)->rr[count_rr].intensity =
				pfs->pr[count].intensity;
			(*rep)->rr[count_rr].pfs_count = 0;
			if (pfs->pr[count].intensity) {
				(*rep)->rr[count_rr].rest_or_work = 1;
				(*rep)->rr[count_rr].pfs_count = 1;
			}
			(*rep)->rr[count_rr].pfs =
				pfs->pr[count].pfs;
			count_rr++;
			continue;
		}
		if ((pfs->pr[count].current_time -
			pfs->pr[count-1].current_time > 62)) {
			(*rep)->rr[count_rr].start_time =
				pfs->pr[count-1].current_time;
			(*rep)->rr[count_rr].stop_time =
				pfs->pr[count].current_time - 60;
			(*rep)->rr[count_rr].intensity = 0;
			(*rep)->rr[count_rr].rest_or_work = 0;
			(*rep)->rr[count_rr].pfs = 0.0;
			(*rep)->rr[count_rr].pfs_count = 0;
			count_rr++;
			if (count_rr == (*rep)->len) break;
		}
		(*rep)->rr[count_rr].start_time =
			pfs->pr[count].current_time - 60;
		(*rep)->rr[count_rr].stop_time =
			pfs->pr[count].current_time;
		(*rep)->rr[count_rr].intensity =
			pfs->pr[count].intensity;
		if (pfs->pr[count].intensity > 0) {
			(*rep)->rr[count_rr].rest_or_work = 1;
			(*rep)->rr[count_rr].pfs = pfs->pr[count].pfs;
			(*rep)->rr[count_rr].pfs_count = 1;
		}
		else {
			(*rep)->rr[count_rr].rest_or_work = 0;
			(*rep)->rr[count_rr].pfs = 0.0;
			(*rep)->rr[count_rr].pfs_count = 0;
		}
		count_rr++;
		if (count_rr == (*rep)->len) break;
	}

	merge_rest_record(*rep);

	merge_work_record(*rep);

	//~ merge_Xminutlimit(*rep, MIN_REST_LENGTH);

	merge_period_record(*rep);
}

void merge_period_record(struct REPORT * rep) {
	int count_rr = 1;
	do {
		if (rep->rr[count_rr].start_time == 0) break;
		if ((rep->rr[count_rr].intensity > 0) &&
		(rep->rr[count_rr-1].intensity > 0)) {
			rep->rr[count_rr].start_time =
				rep->rr[count_rr-1].start_time;
			rep->rr[count_rr].stop_time =
				rep->rr[count_rr].stop_time;
			rep->rr[count_rr].intensity +=
				rep->rr[count_rr-1].intensity;
			rep->rr[count_rr].rest_or_work = 1;
			if (rep->rr[count_rr-1].pfs_count == 0)
				rep->rr[count_rr-1].pfs_count = 1;
			if (rep->rr[count_rr].pfs_count == 0)
				rep->rr[count_rr].pfs_count = 1;
			rep->rr[count_rr].pfs = rep->rr[count_rr].pfs *
				rep->rr[count_rr].pfs_count +
				rep->rr[count_rr-1].pfs *
				rep->rr[count_rr-1].pfs_count;
			rep->rr[count_rr].pfs_count +=
				rep->rr[count_rr-1].pfs_count;
			rep->rr[count_rr].pfs /= (double)
				rep->rr[count_rr].pfs_count;
			delete_1struct(rep, count_rr-1);
			continue;
		}

		if ((rep->rr[count_rr].stop_time -
		rep->rr[count_rr].start_time)/60 < MIN_REST_LENGTH &&
		rep->rr[count_rr].intensity == 0 &&
		rep->rr[count_rr-1].intensity > 0) {
			rep->rr[count_rr].start_time =
				rep->rr[count_rr-1].start_time;
			rep->rr[count_rr].stop_time =
				rep->rr[count_rr].stop_time;
			rep->rr[count_rr].intensity =
				rep->rr[count_rr-1].intensity;
			rep->rr[count_rr].rest_or_work =
				rep->rr[count_rr-1].rest_or_work;
			rep->rr[count_rr].pfs =
				rep->rr[count_rr-1].pfs;
			rep->rr[count_rr].pfs_count =
				rep->rr[count_rr-1].pfs_count;
			delete_1struct(rep, count_rr-1);
			continue;
		}
		count_rr++;
	}
	while (count_rr < rep->len);
}

void delete_1struct(struct REPORT * rep, int number) {
	int rec_move = rep->len - number;
	int mem_size = rec_move * sizeof(struct REPORT_RECORD);
	memmove(&rep->rr[number], &rep->rr[number+1], mem_size);
	memset(&rep->rr[rep->len-1], 0, sizeof(struct REPORT_RECORD));
}

void merge_Xminutlimit(struct REPORT * rep, int limit) {
	if (rep->len < 3) return;
	int count_rr = 2, count = 0;
	do {
		if (rep->rr[count].start_time == 0) break;
		if (rep->rr[count_rr-2].intensity >= MIN_REST_INTENSITY &&
		rep->rr[count_rr-1].intensity < MIN_REST_INTENSITY &&
		rep->rr[count_rr].intensity >= MIN_REST_INTENSITY &&
		(rep->rr[count_rr-1].stop_time -
		rep->rr[count_rr-1].start_time)/60 < limit) {
			rep->rr[count_rr].start_time =
				rep->rr[count_rr-2].start_time;
			rep->rr[count_rr].stop_time =
				rep->rr[count_rr].stop_time;
			rep->rr[count_rr].intensity +=
				rep->rr[count_rr-2].intensity;
			rep->rr[count_rr].rest_or_work = 1;
			if (rep->rr[count_rr-2].pfs_count == 0)
				rep->rr[count_rr-2].pfs_count = 1;
			if (rep->rr[count_rr].pfs_count == 0)
				rep->rr[count_rr].pfs_count = 1;
			rep->rr[count_rr].pfs +=
				rep->rr[count_rr-2].pfs *
				rep->rr[count_rr-2].pfs_count; 
			rep->rr[count_rr].pfs_count +=
				rep->rr[count_rr-2].pfs_count;
			rep->rr[count_rr].pfs /= (double)
				rep->rr[count_rr].pfs_count;
			delete_1struct(rep, count_rr-2);
			delete_1struct(rep, count_rr-2);
		}
		else count_rr++;
		count++;
	}
	while (count_rr < rep->len && count < rep->len);
}

void merge_work_record(struct REPORT * rep) {
	int count_rr = 1, count = 0;
	do {

		if (rep->rr[count_rr].start_time == 0) break;
		if ((rep->rr[count_rr].intensity > 0) &&
		(rep->rr[count_rr-1].intensity > 0) &&
		(rep->rr[count_rr-1].stop_time - rep->rr[count_rr-1].start_time)
		> 59) {
			rep->rr[count_rr].start_time =
				rep->rr[count_rr-1].start_time;
			rep->rr[count_rr].stop_time =
				rep->rr[count_rr].stop_time;
			rep->rr[count_rr].intensity +=
				rep->rr[count_rr-1].intensity;
			rep->rr[count_rr].rest_or_work = 1;
			if (rep->rr[count_rr-1].pfs_count == 0)
				rep->rr[count_rr-1].pfs_count = 1;
			if (rep->rr[count_rr].pfs_count == 0)
				rep->rr[count_rr].pfs_count = 1;
			rep->rr[count_rr].pfs +=
				rep->rr[count_rr-1].pfs *
				rep->rr[count_rr-1].pfs_count; 
			rep->rr[count_rr].pfs_count +=
				rep->rr[count_rr-1].pfs_count;
			rep->rr[count_rr].pfs /= (double)
				rep->rr[count_rr].pfs_count;
			delete_1struct(rep, count_rr-1);
		}
		else count_rr++;
		if (rep->rr[count].intensity > 0 &&
			rep->rr[count].pfs_count == 0)
				rep->rr[count].pfs_count = 1;
		count++;
	}
	while (count_rr < rep->len && count < rep->len);
}

void merge_rest_record(struct REPORT * rep) {
	int count_rr = 1, count = 0;
	do {
		if (rep->rr[count_rr].start_time == 0) break;
		if ((rep->rr[count_rr].intensity <
			MIN_REST_INTENSITY) &&
			(rep->rr[count_rr-1].intensity <
			MIN_REST_INTENSITY)) {
				rep->rr[count_rr].start_time =
					rep->rr[count_rr-1].start_time;
				rep->rr[count_rr].stop_time =
					rep->rr[count_rr].stop_time;
				rep->rr[count_rr].intensity = 0;
				rep->rr[count_rr].rest_or_work = 0;
				rep->rr[count_rr].pfs = 0;
				rep->rr[count_rr].pfs_count = 0;
				delete_1struct(rep, count_rr-1);
		}
		else count_rr++;
		count++;
	}
	while (count_rr < rep->len && count < rep->len);
}

int when_a_break (struct PFS * pfs, time_t current_time) {
	if (!pfs->len) return 90;
	int count = 0;
	for (count = pfs->len; count > -1; count--) {
		if (pfs->pr[count].current_time) break;
	}

	if (count < 0) return 90;

	int time_break = 0;
	int time_rest = 0;
	time_t old_current_time = pfs->pr[count].current_time;
	for ( ; count > -1; count--) {
		if (old_current_time - pfs->pr[count].current_time >
		65) {
			time_rest += (old_current_time -
				pfs->pr[count].current_time) / 60;
		}
		old_current_time = pfs->pr[count].current_time;
		if (pfs->pr[count].intensity < MIN_REST_INTENSITY)
			time_rest++;
		if (time_rest > MIN_REST_LENGTH) break;
		if (pfs->pr[count].intensity > MIN_REST_INTENSITY) {
			time_break += time_rest;
			time_rest = 0;
			time_break++;
		}
	}
	return (90 - time_break);
}

void warning (char type) {
	char * buffer_display = calloc(500, sizeof(char));
	strcpy(buffer_display, "DISPLAY=:0 notify-send \"abashin.ru\" \"");

	char * buffer_speak = calloc(500, sizeof(char));
	strcpy(buffer_speak, "espeak  -vru -s130 \"");

	switch (type) {
		case 1: {
			strcat(buffer_speak, "5 minutes before the rest\"");
			strcat(buffer_display, "5 minutes before the rest\"");
			break;
		}
		case 2: {
			strcat(buffer_speak, "Warning. Rest. Repeat. Warning. Rest.\"");
			strcat(buffer_display, "Warning. Rest. Repeat. Warning. Rest.\"");
			break;
		}
		case 3: {
			strcat(buffer_speak, "The break is over.\"");
			strcat(buffer_display, "The break is over.\"");
			break;
		}
		default: strcat(buffer_speak, "Work\"");
	}
	int result = system(buffer_display);
	if (!result) {
		free(buffer_display);
		printf("Предупреждение не отображено!/n");
		return;
	}
}

char * min_to_hm (const time_t period, char * buffer) {
	time_t per = period;
	time_t temp = per / 86400;
	buffer[0] = 0;
	if (temp > 0) {
		sprintf(buffer,"%ldд. ",temp);
		per -= 86400 * temp;
	}
	char buf[50] = {0};
	temp = per / 3600;
	if (temp > 0) {
		sprintf(buf,"%ldч. ",temp);
		strcat(buffer, buf);
		per -= 3600 * temp;
	}
	temp = per / 60;
	if (temp > 0) {
		sprintf(buf,"%ldм. ",temp);
		strcat(buffer, buf);
		per -= 60 * temp;
	}
	temp = per;
	if (temp > 0) {
		sprintf(buf,"%ldс.",temp);
		strcat(buffer, buf);
		per -= 60 * temp;
	}
	return buffer;
}

void get_earth_day (time_t current_time, time_t * begin_time, time_t * end_time) {
	struct tm * timeinfo;
	timeinfo = localtime(&current_time);
	timeinfo->tm_hour = 0;
	timeinfo->tm_min = 0;
	timeinfo->tm_sec = 0;
	*begin_time = mktime (timeinfo);
	timeinfo->tm_hour = 23;
	timeinfo->tm_min = 59;
	timeinfo->tm_sec = 59;
	*end_time = mktime (timeinfo);
}
