/*Copyright (C) 2020 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include "AbashinRuStatus.h"


void calculate_pfs(struct PFS_RECORD * pfsr_previous,
	struct PFS_RECORD * pfsr_current, struct MOUSE_ETALON * me,
	struct KEYBOARD_ETALON * ke, struct MOUSE_PRESS * mp,
	struct KEYBOARD_PRESS * kp) {
	if (!pfsr_current->current_time)
		pfsr_current->current_time = time(NULL);
	pfsr_current->device_id = 0;
	pfsr_current->intensity = 0;
	pfsr_current->pfs = 0.0;
	int intensity_m = 0;
	double pfs_m = 0.0;
	calculate_mouse (pfsr_current, me, mp, &intensity_m, &pfs_m);

	double pfs_k = 0.0;
	int intensity_k = 0;
	calculate_keyboard (pfsr_current, ke, kp, &intensity_k, &pfs_k);

	pfsr_current->device_id = 0;
	pfsr_current->intensity = intensity_m + intensity_k;
	double del = pfsr_current->intensity;
	if (del < 1.0) del = 1.0;
	pfsr_current->pfs = (pfs_m + pfs_k) / del;
}

double calc_sign(int value, int min, int max) {
	if(value > max) return 0.0;
	if(value < min) return 1.0;
	double del = (double)(max - min);
	if (del < 1.0) del = 1.0;
	return 1.0 - (double)(value - min)/del;
}

void calculate_mouse (struct PFS_RECORD * pfsr_current,
	struct MOUSE_ETALON * me, struct MOUSE_PRESS * mp,
	int * intensity_m, double * pfs_m) {
	int counter = 0;
	for(counter = 0; counter < mp->len; counter++) {
		if (!mp->mpr[counter].intensity) continue;
		if(mp->mpr[counter].current_time >
			pfsr_current->current_time - 360) {
			*intensity_m += mp->mpr[counter].intensity;
			int counter_me = 0;
			for(counter_me = 0; counter_me < me->len;
							counter_me++) {
				if(mp->mpr[counter].key !=
					me->mer[counter_me].key)
					continue;
				double pfs_mt = calc_sign(mp->mpr[counter].retaining1,
					me->mer[counter_me].retaining1_min,
					me->mer[counter_me].retaining1_max);
				if (pfs_mt != 0.0)
					*pfs_m += pfs_mt * (double)mp->mpr[counter].intensity;
				break;
			}
		}
	}
}

void calculate_keyboard (struct PFS_RECORD * pfsr_current,
	struct KEYBOARD_ETALON * ke, struct KEYBOARD_PRESS * kp,
	int * intensity_k, double * pfs_k) {
	int counter = 0;
	for(counter = 0; counter < kp->len; counter++) {
		if(kp->kpr[counter].current_time >
			pfsr_current->current_time - 360) {
			*intensity_k += kp->kpr[counter].intensity;
			int counter_ke = 0;
			for(counter_ke = 0; counter_ke < ke->len;
							counter_ke++) {
				if(kp->kpr[counter].key1 !=
					ke->ker[counter_ke].key1 ||
					kp->kpr[counter].key2 !=
					ke->ker[counter_ke].key2)
						continue;
				double pfs_kr1 = calc_sign(kp->kpr[counter].retaining1,
					ke->ker[counter_ke].retaining1_min,
					ke->ker[counter_ke].retaining1_max);
				double pfs_ks = calc_sign(kp->kpr[counter].space1,
					ke->ker[counter_ke].space1_min,
					ke->ker[counter_ke].space1_max);
				double pfs_kr2 = calc_sign(kp->kpr[counter].retaining2,
					ke->ker[counter_ke].retaining2_min,
					ke->ker[counter_ke].retaining2_max);

				double pfs_kt = 0.0;
				char c1 = 0;
				if(pfs_kr1) {
					pfs_kt = pfs_kr1;
					c1++;
				}
				if(pfs_ks) {
					pfs_kt = pfs_ks;
					c1++;
				}
				if(pfs_kr2) {
					pfs_kt = pfs_kr2;
					c1++;
				}
				pfs_kt = pfs_kt / (double)c1;

				if (pfs_kt > 0.0)
					*pfs_k += pfs_kt * (double)kp->kpr[counter].intensity;
				break;
			}
		}
	}
}
