/*Copyright (C) 2020 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include "libdevice.h"

#define true 1
#define false 0

#define KEYBOARDSPATH "/dev/input/"
#define KEYBOARDPATHSIZE 100
#define TEMPFILENAMESIZE 50
#define TEMPFILENAME "/tmp/abashin.ru_keyboards"
#define ETALONFILENAME "/tmp/abashin.ru_keyboards_etalon"
#define FINDLOOPDURATION 60

#define PROC_FILE_PATH "/proc/bus/input/devices"
#define DEVICES_PATH_SIZE 20
#define DEVICES_COUNT 10


struct keyboard {
	pid_t pid;
	char keyboard_path[KEYBOARDPATHSIZE];
};

int reset_disabled_keyboards(struct keyboard * keyboards);
int find_keyboards_and_start(char * device_path, char * tempfilename,
				struct keyboard * keyboards);
char * get_keyboards(char * k_massive);

void create_etalon(char * tempfilename);

int main(int argc, char * argv[]) {
	struct keyboard keyboard_massive [DEVICES_COUNT] = {0};

	char tempfilename [TEMPFILENAMESIZE] = {0};
	create_temp_file(TEMPFILENAME, tempfilename);

	start_signal_handler();

	while(1) {
		int result = 0;
		result = reset_disabled_keyboards((struct keyboard *)
			&keyboard_massive);
		if (result) fprintf(stderr, "Can`t reset keyboards.\n");
		char path[100] = {0};
		strcpy(path, KEYBOARDSPATH);
		result = find_keyboards_and_start (path, tempfilename,
				(struct keyboard *) &keyboard_massive);
		if (result) fprintf(stderr, "Can`t find and start keyboards.\n");
		sleep(FINDLOOPDURATION);

		create_etalon(tempfilename);

		send_data();

		chmod(ETALONFILENAME, 0744);

		reset_file(tempfilename);
	}
	return 0;
}

void create_etalon(char * tempfilename) {
	pid_t pid = fork();
	if (pid < 0) {
		fprintf(stderr, "Fork create etalon failed\n");
		exit(-1);
	}
	else
		if (pid == 0) {
			execlp("/usr/sbin/keyboard_etalon",
			"keyboard_etalon", tempfilename,
			ETALONFILENAME, NULL);
		}
		else {
			int status;
			wait(&status);
		}
}

int reset_disabled_keyboards(struct keyboard * keyboards) {
	int count;
	for (count = 0; count < DEVICES_COUNT; count++) {
		if (keyboards[count].pid != 0) {
			FILE * keyboard_file = fopen((char *)&keyboards[count].keyboard_path,"r");
			if (keyboard_file == NULL) {
				kill (keyboards[count].pid, SIGKILL);
				bzero(&keyboards[count].keyboard_path,
				sizeof(char)*KEYBOARDPATHSIZE);
			}
			else fclose (keyboard_file);
			if (kill (keyboards[count].pid, 0) != 0) {
				keyboards[count].pid = 0;
				bzero(&keyboards[count].keyboard_path,
				sizeof(char)*KEYBOARDPATHSIZE);
			}
		}
	}
	return 0;
}

int find_keyboards_and_start(char * device_path, char * tempfilename,
                             struct keyboard * keyboards) {
	char * k_massive = calloc(sizeof(char),
				DEVICES_PATH_SIZE*DEVICES_COUNT);
	bzero(k_massive, DEVICES_PATH_SIZE*DEVICES_COUNT*sizeof(char));
	get_keyboards (k_massive);

	int count = 0;
	for (count=0; count < DEVICES_COUNT; ++count) {
		if (k_massive[count*DEVICES_PATH_SIZE]) {
			char * keyboard_path = calloc(sizeof(char),
						KEYBOARDPATHSIZE);
			strcpy(keyboard_path, device_path);
			strcat(keyboard_path,
				(char*)&k_massive[count*DEVICES_PATH_SIZE]
			);
			char find_flag = 0;
			int count = 0;
			for (count = 0; count < DEVICES_COUNT; count++) {
				if (strcmp (keyboard_path,
				(char *)&keyboards[count].keyboard_path) == 0) {
					find_flag = 1;
					break;
				}
			}
			if (find_flag == 0) {
				for (count = 0; count < DEVICES_COUNT;
				count++) {
					if (keyboards[count].pid == 0) {
						strcpy((char *)
						&keyboards[count].keyboard_path,
						keyboard_path
						);
						pid_t pid = fork();
						if (pid < 0) {
							fprintf(stderr,
							"Fork new keyboard failed\n"
							);
							exit(-1);
						}
						else
							if (pid == 0) {
								execlp("/usr/sbin/keyboard_client",
								"keyboard_client",
								(char *)&keyboards[count].keyboard_path,
								tempfilename, NULL
								);
							}
							else keyboards[count].pid = pid;
						break;
					}
				}
			}
			free (keyboard_path);
		}
	}
	free (k_massive);
	return 0;
}

char * get_keyboards(char * k_massive) {
	size_t READ_B_SIZE = 200;
	FILE * source_file;
	char read_flag = 1;
	int k_massive_count = 0;
	source_file = fopen(PROC_FILE_PATH,"r");
	if (source_file == NULL) return NULL;
	char * read_buffer = calloc(sizeof(char),READ_B_SIZE+1);
	while(1) {
		if (!getline(&read_buffer, &READ_B_SIZE, source_file))
			break;
		if (feof(source_file)) break;
		if (read_flag) {
			if ((read_buffer[0] == 'N') &&
			(!strstr(read_buffer, "eyboard"))) {
				read_flag = 0;
				continue;
			}
			if (read_buffer[0] == 'H') {
				char event[10] = {'e','v','e','n','t',0};
				char * ptr = strstr(read_buffer,"event");
				if (ptr) {
					int temp_count = 0;
					ptr+=5;
					for (temp_count = 0;
					temp_count < 5; ++temp_count) {
						if (ptr[temp_count] == ' ')
							break;
						event[temp_count+5] =
							ptr[temp_count];
					}
					strcpy(&k_massive[k_massive_count*
						DEVICES_PATH_SIZE],
						&event[0]
					);
					k_massive_count++;
				}
			}
		}
		if (read_buffer[0] == 0x0A) read_flag = 1;
	}
	fclose(source_file);
	free(read_buffer);
	return k_massive;
}
