/*Copyright (C) 2020 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include "libdevice.h"

#define true 1
#define false 0

#define MOUSESPATH "/dev/input/"
#define MOUSES 5
#define MOUSEPATHSIZE 100
#define TEMPFILENAMESIZE 50
#define TEMPFILENAME "/tmp/abashin.ru_mouses"
#define ETALONFILENAME "/tmp/abashin.ru_mouses_etalon"
#define FINDLOOPDURATION 60

#define PROC_FILE_PATH "/proc/bus/input/devices"
#define DEVICES_PATH_SIZE 20
#define DEVICES_COUNT 10

sig_atomic_t child_exit_status;
pthread_t ptwaiter_id;
int sig_num;

struct mouse {
	pid_t pid;
	char mouse_path[MOUSEPATHSIZE];
};

int reset_disabled_mouses(struct mouse * mouses);
int find_mouses_and_start (char * device_path, char * tempfilename,
			struct mouse * mouses);
char * get_mouses_touchpads (char * k_massive);
void create_etalon(char * tempfilename);

int main(int argc, char * argv[]) {
	struct mouse mouse_massive [MOUSES] = {0};

	char tempfilename [TEMPFILENAMESIZE] = {0};
	create_temp_file(TEMPFILENAME, tempfilename);

	start_signal_handler();

	while(1) {
		int result = 0;
		result = reset_disabled_mouses((struct mouse *)
				&mouse_massive);
		if (result) fprintf(stderr, "Can`t reset mouses.\n");
		char path[100] = {0};
		strcpy((char *)&path, MOUSESPATH);
		result = find_mouses_and_start ((char *)&path,
			(char *)&tempfilename, (struct mouse *) &mouse_massive);
		if (result)
			fprintf(stderr, "Can`t find and start mouses.\n");
		sleep(FINDLOOPDURATION);

		create_etalon(tempfilename);

		send_data();

		chmod(ETALONFILENAME, 0744);

		reset_file(tempfilename);
	}
	return 0;
}

void create_etalon(char * tempfilename) {
	pid_t pid = fork();
	if (pid < 0) {
		fprintf(stderr, "Fork create etalon mouses failed\n");
		exit(-1);
	}
	else
		if (pid == 0) {
			execlp("/usr/sbin/mouse_etalon","mouse_etalon",
				tempfilename, ETALONFILENAME, NULL);
		}
		else {
			int status;
			wait(&status);
		}
}

int reset_disabled_mouses(struct mouse * mouses) {
  int count;
  for (count = 0; count < MOUSES; count++) {
    if (mouses[count].pid != 0) {
      FILE * mouse_file = fopen((char *)&mouses[count].mouse_path,"r");
      if (mouse_file == NULL) {
        kill (mouses[count].pid, SIGKILL);
        bzero(&mouses[count].mouse_path, sizeof(char)*MOUSEPATHSIZE);
      }
      else fclose (mouse_file);
      if (kill (mouses[count].pid, 0) != 0) {
        mouses[count].pid = 0;
        bzero(&mouses[count].mouse_path, sizeof(char)*MOUSEPATHSIZE);
      }
    }
  }
  return 0;
}

int find_mouses_and_start(char * device_path, char * tempfilename,
				struct mouse * mouses) {
	char * k_massive = calloc(sizeof(char),
				DEVICES_PATH_SIZE*DEVICES_COUNT);
	bzero(k_massive, DEVICES_PATH_SIZE*DEVICES_COUNT*sizeof(char));
	get_mouses_touchpads(k_massive);

	int count = 0;
	for (count = 0; count < DEVICES_COUNT; count++) {
		if (k_massive[count*DEVICES_PATH_SIZE]){
			char * mouse_path = calloc(sizeof(char),
							MOUSEPATHSIZE);
			strcpy(mouse_path, device_path);
			strcat(mouse_path,
				(char*)&k_massive[count*DEVICES_PATH_SIZE]);
			char find_flag = 0;
			int hidden_count = 0;
			for (hidden_count = 0; hidden_count < MOUSES;
				hidden_count++) {
				if (strcmp (mouse_path,
					mouses[hidden_count].mouse_path) == 0) {
					find_flag = 1;
					break;
				}
			}
			if (find_flag == 0) {
				for (hidden_count = 0;
				hidden_count < MOUSES; hidden_count++) {
					if (mouses[hidden_count].pid == 0) {
						strcpy(mouses[hidden_count].mouse_path,
							mouse_path);
						pid_t pid = fork();
						if (pid < 0) {
							fprintf(stderr, "Fork new mouse failed\n");
							exit(-1);
						}
						else
							if (pid == 0) {
								execlp("/usr/sbin/mouse_client",
								"mouse_client",
								mouses[hidden_count].mouse_path,
								tempfilename, NULL);
							}
							else {
								mouses[hidden_count].pid = pid;
							}
							break;
						}
					}
				}
			free(mouse_path);
		}
		else break;
	}
	free(k_massive);
	return 0;
}

char * get_mouses_touchpads(char * k_massive) {
	size_t READ_B_SIZE = 200;
	FILE * source_file;
	char read_flag = 1;
	int k_massive_count = 0;
	source_file = fopen(PROC_FILE_PATH,"r");
	if (source_file == NULL) return NULL;
	char * read_buffer = calloc(sizeof(char),READ_B_SIZE+1);
	while(getline(&read_buffer, &READ_B_SIZE, source_file) != -1) {
		if (read_flag) {
			if (read_buffer[0] == 'N')
				if ((!strstr(read_buffer, "ouse")) &&
					(!strstr(read_buffer, "OUSE")) &&
					(!strstr(read_buffer, "ouchPad")) &&
					(!strstr(read_buffer, "ouchpad"))) {
					read_flag = 0;
					continue;
				}
			if (read_buffer[0] == 'H') {
				char event[10] = {'e','v','e','n','t',0};
				char * ptr = strstr(read_buffer, "event");
				if (ptr) {
					int temp_count = 0;
					ptr+=5;
					for (temp_count = 0;
					temp_count < 5; ++temp_count) {
						if (ptr[temp_count] == ' ')
							break;
						event[temp_count+5] =
							ptr[temp_count];
					}
					strcpy(&k_massive[k_massive_count*DEVICES_PATH_SIZE],
                                        &event[0]);
					if (k_massive_count >= DEVICES_COUNT-1)
						break;
					k_massive_count++;
				}
			}
		}
		if (read_buffer[0] == 0x0A) read_flag = 1;
		bzero(read_buffer,READ_B_SIZE+1);
	}
	fclose(source_file);
	free (read_buffer);
	return k_massive;
}
